# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: openocd.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='openocd.proto',
  package='rpi',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\ropenocd.proto\x12\x03rpi\"$\n\x10LogStreamRequest\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\"!\n\x11LogStreamResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\"\x1e\n\x0eProgramRequest\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\x06\n\x04void2\x8a\x02\n\x07OpenOcd\x12@\n\rProgramDevice\x12\x13.rpi.ProgramRequest\x1a\x16.rpi.LogStreamResponse(\x01\x30\x01\x12\x32\n\x0bResetDevice\x12\t.rpi.void\x1a\x16.rpi.LogStreamResponse0\x01\x12\"\n\nStartDebug\x12\t.rpi.void\x1a\t.rpi.void\x12!\n\tStopDebug\x12\t.rpi.void\x1a\t.rpi.void\x12\x42\n\x0fLogStreamCreate\x12\x15.rpi.LogStreamRequest\x1a\x16.rpi.LogStreamResponse0\x01\x62\x06proto3'
)




_LOGSTREAMREQUEST = _descriptor.Descriptor(
  name='LogStreamRequest',
  full_name='rpi.LogStreamRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='filename', full_name='rpi.LogStreamRequest.filename', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22,
  serialized_end=58,
)


_LOGSTREAMRESPONSE = _descriptor.Descriptor(
  name='LogStreamResponse',
  full_name='rpi.LogStreamResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='rpi.LogStreamResponse.data', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=93,
)


_PROGRAMREQUEST = _descriptor.Descriptor(
  name='ProgramRequest',
  full_name='rpi.ProgramRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='rpi.ProgramRequest.data', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=95,
  serialized_end=125,
)


_VOID = _descriptor.Descriptor(
  name='void',
  full_name='rpi.void',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=127,
  serialized_end=133,
)

DESCRIPTOR.message_types_by_name['LogStreamRequest'] = _LOGSTREAMREQUEST
DESCRIPTOR.message_types_by_name['LogStreamResponse'] = _LOGSTREAMRESPONSE
DESCRIPTOR.message_types_by_name['ProgramRequest'] = _PROGRAMREQUEST
DESCRIPTOR.message_types_by_name['void'] = _VOID
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LogStreamRequest = _reflection.GeneratedProtocolMessageType('LogStreamRequest', (_message.Message,), {
  'DESCRIPTOR' : _LOGSTREAMREQUEST,
  '__module__' : 'openocd_pb2'
  # @@protoc_insertion_point(class_scope:rpi.LogStreamRequest)
  })
_sym_db.RegisterMessage(LogStreamRequest)

LogStreamResponse = _reflection.GeneratedProtocolMessageType('LogStreamResponse', (_message.Message,), {
  'DESCRIPTOR' : _LOGSTREAMRESPONSE,
  '__module__' : 'openocd_pb2'
  # @@protoc_insertion_point(class_scope:rpi.LogStreamResponse)
  })
_sym_db.RegisterMessage(LogStreamResponse)

ProgramRequest = _reflection.GeneratedProtocolMessageType('ProgramRequest', (_message.Message,), {
  'DESCRIPTOR' : _PROGRAMREQUEST,
  '__module__' : 'openocd_pb2'
  # @@protoc_insertion_point(class_scope:rpi.ProgramRequest)
  })
_sym_db.RegisterMessage(ProgramRequest)

void = _reflection.GeneratedProtocolMessageType('void', (_message.Message,), {
  'DESCRIPTOR' : _VOID,
  '__module__' : 'openocd_pb2'
  # @@protoc_insertion_point(class_scope:rpi.void)
  })
_sym_db.RegisterMessage(void)



_OPENOCD = _descriptor.ServiceDescriptor(
  name='OpenOcd',
  full_name='rpi.OpenOcd',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=136,
  serialized_end=402,
  methods=[
  _descriptor.MethodDescriptor(
    name='ProgramDevice',
    full_name='rpi.OpenOcd.ProgramDevice',
    index=0,
    containing_service=None,
    input_type=_PROGRAMREQUEST,
    output_type=_LOGSTREAMRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ResetDevice',
    full_name='rpi.OpenOcd.ResetDevice',
    index=1,
    containing_service=None,
    input_type=_VOID,
    output_type=_LOGSTREAMRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StartDebug',
    full_name='rpi.OpenOcd.StartDebug',
    index=2,
    containing_service=None,
    input_type=_VOID,
    output_type=_VOID,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StopDebug',
    full_name='rpi.OpenOcd.StopDebug',
    index=3,
    containing_service=None,
    input_type=_VOID,
    output_type=_VOID,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='LogStreamCreate',
    full_name='rpi.OpenOcd.LogStreamCreate',
    index=4,
    containing_service=None,
    input_type=_LOGSTREAMREQUEST,
    output_type=_LOGSTREAMRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_OPENOCD)

DESCRIPTOR.services_by_name['OpenOcd'] = _OPENOCD

# @@protoc_insertion_point(module_scope)
